@@ -21,19 +21,29 @@
 #---------------------------------------------------------------------------
 from base_device_module import deviceModule
 
-def getModule(m,d):
-  return(device_neo(m,d))
+def getModule(m,d,i):
+  return(device_neo(m,d,i))
 
 class device_neo(deviceModule):
   """A Neo FreeRunner modRana device-specific module"""
   
-  def __init__(self, m, d):
-    deviceModule.__init__(self, m, d)
+  def __init__(self, m, d, i):
+    deviceModule.__init__(self, m, d, i)
     self.tempUnfullscreen = False
 
   def getDeviceName(self):
     return "OpenMoko Neo FreeRunner"
 
+  def simpleMapDragging(self):
+    return True
+
+  def lpSkipCount(self):
+    """SHR on Neo fires two clicks after a long press, so we need to skip
+    bot of them, to avoid clicking some new button that
+    shows up after the screen redraws"""
+    return 2
+
+
   def textEntryIminent(self):
     """in SHR on Neo, we need to temporarry disable fullscreen
     (if we are in fullscreen),
@@ -53,8 +63,6 @@ class device_neo(deviceModule):
           display.fullscreenToggle()
           self.tempUnfullscreen = False
 
-    
-
 if(__name__ == "__main__"):
   a = device_example({}, {})
   a.update()
