@@ -18,17 +18,19 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
 from base_module import ranaModule
-from time import time
 from tilenames import *
 
-def getModule(m,d):
-  return(mapView(m,d))
+def getModule(m,d,i):
+  return(mapView(m,d,i))
 
 class mapView(ranaModule):
   """Controls the view being displayed on the map"""
-  def __init__(self, m, d):
-    ranaModule.__init__(self, m, d)
-    self.lastPos = None
+  def __init__(self, m, d, i):
+    ranaModule.__init__(self, m, d, i)
+
+  def firstTime(self):
+    self.checkMapDraggingMode() # check the map dragging mode on startup
+    self.checkCenteringDisableTreshold() # check centering disable treshold on startup
     
   def handleMessage(self, message, type, args):
     z = self.get('z', 15)
@@ -45,6 +47,12 @@ class mapView(ranaModule):
         (lat,lon) = pos
         proj.recentre(lat, lon, z)
 
+    elif message=="dragModeChanged":
+      self.checkMapDraggingMode()
+
+    elif message=="centeringDisableTresholdChanged":
+      self.checkCenteringDisableTreshold()
+
     elif(message):
       try:
         list = message.split(' ')
@@ -61,16 +69,8 @@ class mapView(ranaModule):
         print "mapView: cant recentre cooridnates"
   
   def dragEvent(self,startX,startY,dx,dy,x,y):
-    # check if centering is on
-    if self.get("centred",True):
-      fullDx = x - startX
-      fullDy = y - startY
-      distSq = fullDx * fullDx + fullDy * fullDy
-      """ check if the drag is strong enought to disable centering
-      -> like this, centering is not dsabled by pressing buttons"""
-      if distSq > 1024:
-        self.set("centred",False) # turn off centering after dragging the map (like in TangoGPS)
-    else:
+    # only drag the map if centering is disabled
+    if not self.get("centred",False):
       proj = self.m.get('projection', None)
       if proj:
         proj.nudge(dx,dy)
@@ -82,14 +82,8 @@ class mapView(ranaModule):
       # get current position information
       pos = self.get('pos', None)
       # check if the position changed from last time
-      if pos != self.lastPos:
-        if(self.setCentre(pos)):
-          self.set('centreOnce', False)
-        self.lastPos = pos
-        
-#    request = self.get("centreOn", None)
-#    if(request):
-#      self.setCentre([float(a) for a in request.split(",")])
+      if(self.setCentre(pos)):
+        self.set('centreOnce', False)        
       
   def setCentre(self,pos):
     """takes care for centering the map on current position"""
@@ -112,4 +106,21 @@ class mapView(ranaModule):
       """
       proj.recentre(lat,lon,z)
       self.set("needRedraw", True)
-      return(True)
\ No newline at end of file
+      return(True)
+
+    # * map draging mode control * #
+  def checkMapDraggingMode(self):
+    """check and set current redraw mode configuration"""
+    draggingMode = self.get('mapDraggingMode', "default")
+    print "mapView: switching map drag mode to %s" % draggingMode
+    if draggingMode == 'default':
+      self.modrana.setDefaultDrag()
+    elif draggingMode == "staticMapDrag":
+      self.modrana.staticMapDragEnable()
+
+  def checkCenteringDisableTreshold(self):
+    """check ans set current centering disable treshold"""
+    centeringDisableTreshold = self.get('centeringDisableTreshold', 2048)
+    print "mapView: switching centering disable treshold to %s" % centeringDisableTreshold
+    self.modrana.setCDDragTreshold(int(centeringDisableTreshold))
+
