@@ -18,19 +18,20 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
 from base_module import ranaModule
-import cairo
 import marshal
 
-def getModule(m,d):
-  return(options(m,d))
+def getModule(m,d,i):
+  return(options(m,d,i))
 
 class options(ranaModule):
   """Handle options"""
-  def __init__(self, m, d):
-    ranaModule.__init__(self, m, d)
+  def __init__(self, m, d, i):
+    ranaModule.__init__(self, m, d, i)
     self.options = {}
     self.scroll = 0
     self.load()
+    self.on = '<span color="green">ON</span>'
+    self.off = '<span color="red">OFF</span>'
   
 
 
@@ -53,12 +54,12 @@ class options(ranaModule):
     id = "opt_group_%s_%s" % (id,parrentId) # get a standardized id
     action="%sset:menu:%s%s" % (actionPrefix,id,actionSufix)
     self.menuModule.addItem(parrentId, name, icon, action)
-    self.options[id] = ("set:menu:%s" % parrentId,[])
+    self.options[id] = ["set:menu:%s" % parrentId,0,[]]
     return id
 
   def addBoolOption(self, title, variable, group, default=None, action=None):
-    on = '<span color="green">ON</span>'
-    off = '<span color="red">OFF</span>'
+    on = self.on
+    off = self.off
     if action:
       self.addOption(title,variable,((False,off,action),(True,on,action)),group,default)
     else:
@@ -67,7 +68,7 @@ class options(ranaModule):
   def addOption(self, title, variable, choices, group, default=None):
     newOption = (title,variable, choices,group,default)
     if self.options.has_key(group):
-      self.options[group][1].append(newOption)
+      self.options[group][2].append(newOption)
     else:
       print "options: group %s does not exist, call addGroup to create it first" % group
 #      self.options[group] = [newOption,]
@@ -140,7 +141,7 @@ class options(ranaModule):
          ("up","shift up"),
          ("left","shift left"),
          ("right","shift right"),
-         (None,"don't shift")],
+         (False,"don't shift")],
          group,
          "down")
 
@@ -152,6 +153,30 @@ class options(ranaModule):
          group,
          0.75)
 
+      changedMsg = "mapView:centeringDisableTresholdChanged"
+      addOpt("Disable by dragging", "centeringDisableTreshold",
+        [(2048,"normal drag - <i>default</i>",changedMsg),
+         (15000,"long drag",changedMsg),
+         (40000,"realy long drag",changedMsg),
+         (80000,"extremely long drag",changedMsg),
+         (False,self.off,changedMsg)],
+         group,
+         2048)
+
+      # ** dragging
+      group = addGroup("Dragging", "dragging", catMap, "generic")
+      # check if we are on a powerful device or not and set the default accordingly
+      if self.dmod.simpleMapDragging():
+        defaultMode = "staticMapDrag"
+      else:
+        defaultMode = "default"
+
+      addOpt("Map dragging", "mapDraggingMode",
+        [("default","full redraw - <i>default</i>","mapView:dragModeChanged"),
+         ("staticMapDrag","drag visible map - <i>fastest</i>", "mapView:dragModeChanged")],
+         group,
+         defaultMode)
+
       # ** tile storage
       group = addGroup("Tile storage", "tile_storage", catMap, "generic")
       addOpt("Tile storage", "tileStorageType",
@@ -308,7 +333,7 @@ class options(ranaModule):
 
     addOpt("read Cyrillic with:", "voiceNavigationCyrillicVoice",
       [('ru',"Russian voice"),
-       (None,"current voice")],
+       (False,"current voice")],
        group,
        'ru')
 
@@ -379,24 +404,6 @@ class options(ranaModule):
        group,
        'knotsPerSecond')
 
-    # * screen *
-    """only add if supported on device"""
-    display = self.m.get('display', None)
-    if display:
-      if display.screenBlankingControlSupported():
-        addOpt("Keep display ON", "screenBlankingMode",
-        [("always", "always", "display:blankingModeChanged"),
-         ("centred", "while centred", "display:blankingModeChanged"),
-         ("moving", "while moving", "display:blankingModeChanged"),
-         ("movingInFullscreen", "while moving in fullscreen", "display:blankingModeChanged"),
-         ("fullscreen", "while in fullscreen", "display:blankingModeChanged"),
-         ("gpsFix", "while there is a GPS fix", "display:blankingModeChanged"), #TODO: while there is actually a GPS lock
-         ("never", "never", "display:blankingModeChanged")],
-         "Display",
-         "always")
-      if display.usesDashboard():
-        addBoolOpt("Redraw when on dashboard", "redrawOnDashboard", 'Display', False)
-
     # * the Network category *
     catNetwork = addCat("Network", "network", "network")
     # * network *
@@ -451,7 +458,7 @@ class options(ranaModule):
 
     # ** redraw
     group = addGroup("Redrawing", "redrawing", catDebug, "generic")
-    addBoolOpt("Print redraw time to terminal", "showRedrawTime", group, False)
+    addBoolOpt("Print redraw time to terminal", "showRedrawTime", group, False, "display:checkShowRedrawTime")
     # ** logging
     group = addGroup("Logging", "logging", catDebug, "generic")
     addBoolOpt("Log modRana stdout to file", "loggingStatus", group, False, "log:checkLoggingStatus")
@@ -470,10 +477,10 @@ class options(ranaModule):
 
 
 #    addOpt("Tracklogs", "showTracklog",
-#    [(None, "Dont draw tracklogs"),
+#    [(False, "Dont draw tracklogs"),
 #     ("simple", "Draw simple tracklogs")],
 #     "view",
-#     None)
+#     False)
 
 
 #    # Add all our categories to the "options" menu
@@ -488,12 +495,13 @@ class options(ranaModule):
 
     # Set all undefined options to default values
     for category,options in self.options.items():
-      for option in options[1]:
+      for option in options[2]:
         (title,variable,choices,category,default) = option
         if(default != None):
           if(not self.d.has_key(variable)):
             self.set(variable, default)
 
+
   def save(self):
     print "options: saving options"
     try:
@@ -535,16 +543,16 @@ class options(ranaModule):
     return("data/options.bin")
   
   def handleMessage(self, message, type, args):
-    if(message == "up"):
-      if(self.scroll > 0):
-        self.scroll -= 1
-        self.set("needRedraw", True)
-    elif(message == "down"):
-      self.scroll += 1
-      self.set("needRedraw", True)
-    elif(message == "reset_scroll"):
-      self.scroll = 0
-      self.set("needRedraw", True)
+    if type=="ml" and message=="scroll":
+      (direction,menuName) = args
+      index = self.options[menuName][1]
+      maxIndex = len(self.options[menuName][2])-1
+      if direction == "up" and index > 0:
+        newIndex = index - 1
+        self.options[menuName][1] = newIndex
+      elif direction == "down" and index < maxIndex:
+        newIndex = index + 1
+        self.options[menuName][1] = newIndex
     elif(message == "save"):
       self.save()
     
@@ -569,20 +577,20 @@ class options(ranaModule):
       (x4,y4) = e4
       (w1,h1,dx,dy) = alloc
 
-      options = self.options[menuName][1]
-      cancelButtonAction = self.options[menuName][0]
+      (cancelButtonAction, firstItemIndex, options) = self.options[menuName]
 
       # Top row:
       # * parent menu
-      self.menuModule.drawButton(cr, x1, y1, dx, dy, "", "up", cancelButtonAction)
+      timeout = self.modrana.msLongPress
+      self.menuModule.drawButton(cr, x1, y1, dx, dy, "", "up", cancelButtonAction, timedAction=(timeout,"set:menu:None"))
       # * scroll up
-      self.menuModule.drawButton(cr, x2, y2, dx, dy, "", "up_list", "options:up")
+      self.menuModule.drawButton(cr, x2, y2, dx, dy, "", "up_list", "ml:options:scroll:up;%s" % menuName)
       # * scroll down
-      self.menuModule.drawButton(cr, x3, y3, dx, dy, "", "down_list", "options:down")
+      self.menuModule.drawButton(cr, x3, y3, dx, dy, "", "down_list", "ml:options:scroll:down;%s" % menuName)
 
       # One option per row
       for row in (0,1,2):
-        index = self.scroll + row
+        index = firstItemIndex + row
         numItems = len(options)
         if(0 <= index < numItems):
           (title,variable,choices,category,default) = options[index]
@@ -648,6 +656,4 @@ class options(ranaModule):
 
 if(__name__ == "__main__"):
   a = options({},{'viewport':(0,0,600,800)})
-  a.firstTime()
-
-  
\ No newline at end of file
+  a.firstTime()
\ No newline at end of file
