@@ -37,13 +37,13 @@ from time import clock
 #from loadOsm import *
 #from route import Router
 
-def getModule(m,d):
-  return(route(m,d))
+def getModule(m,d,i):
+  return(route(m,d,i))
 
 class route(ranaModule):
   """Routes"""
-  def __init__(self, m, d):
-    ranaModule.__init__(self, m, d)
+  def __init__(self, m, d, i):
+    ranaModule.__init__(self, m, d, i)
     self.route = [] # maybe remove this ?
     self.routeRequestSentTimestamp = None
     self.pxpyRoute = []
